/*****************************************************************************
 * Name:      convertergui.cpp
 * Purpose:   Code for Application GUI
 * Created:   2007-04-26
 * Copyright: 2007,2008 Dominik Wagenfuehr <dominik.wagenfuehr@deesaster.org>
 * License:   GPLv3
 *****************************************************************************/

/**
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "convertergui.h"
#include "../library/converterbase.h"
#include <wx/textfile.h>

BEGIN_EVENT_TABLE(ConverterGui, wxFrame)
    EVT_CLOSE(ConverterGui::OnClose)
    EVT_MENU(ID_MENU_QUIT, ConverterGui::OnQuit)
    EVT_MENU(ID_MENU_OPEN, ConverterGui::OnOpen)
    EVT_MENU(ID_MENU_SAVE, ConverterGui::OnSave)
    EVT_MENU(ID_MENU_SAVE_AS, ConverterGui::OnSaveAs)
    EVT_MENU(ID_MENU_ABOUT, ConverterGui::OnAbout)
    EVT_MENU(ID_CONV_HTML2BB, ConverterGui::ConvertHTML2BB)
    EVT_MENU(ID_CONV_HTML2MOIN, ConverterGui::ConvertHTML2Moin)
    EVT_MENU(ID_CONV_BB2HTML, ConverterGui::ConvertBB2HTML)
    EVT_MENU(ID_CONV_BB2MOIN, ConverterGui::ConvertBB2Moin)
    EVT_MENU(ID_CONV_MOIN2HTML, ConverterGui::ConvertMoin2HTML)
    EVT_MENU(ID_CONV_MOIN2BB, ConverterGui::ConvertMoin2BB)
    EVT_TEXT(ID_TEXTFIELD, ConverterGui::OnTextEnter)
END_EVENT_TABLE()

ConverterGui::ConverterGui(wxFrame *frame, const wxString& title, long style)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxDefaultSize, style)
{
    name=_T("HTML/BB/MoinMoin-Converter");
    version=_T("v8.1.12");

    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();

    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(ID_MENU_OPEN, _("&Open...\tCtrl-O"), _("Open new file"));
    fileMenu->Append(ID_MENU_SAVE, _("&Save\tCtrl-S"), _("Save file"));
    fileMenu->Append(ID_MENU_SAVE_AS, _("Save &as...\tCtrl-S"), _("Save file with other filename"));
    fileMenu->AppendSeparator();
    fileMenu->Append(ID_MENU_QUIT, _("&Quit\tAlt-F4"), _("Quit the application"));
    mbar->Append(fileMenu, _("&File"));

    wxMenu* convertMenu = new wxMenu(_T(""));

    wxMenu* convertHtmlMenu = new wxMenu(_T(""));
    convertHtmlMenu->Append(ID_CONV_HTML2BB, _("&bbCode\tF1"), _("Convert to bbCode"));
    convertHtmlMenu->Append(ID_CONV_HTML2MOIN, _("&MoinMoin\tF2"), _("Convert to MoinMoin"));
    convertMenu->Append(wxID_ANY, _("&HTML to..."),convertHtmlMenu);

    wxMenu* convertBBMenu = new wxMenu(_T(""));
    convertBBMenu->Append(ID_CONV_BB2HTML, _("&HTML\tF3"), _("Convert to HTML"));
    convertBBMenu->Append(ID_CONV_BB2MOIN, _("&MoinMoin\tF4"), _("Convert to MoinMoin"));
    convertMenu->Append(wxID_ANY,_("&bbCode to..."), convertBBMenu);

    wxMenu* convertMoinMenu = new wxMenu(_T(""));
    convertMoinMenu->Append(ID_CONV_MOIN2HTML, _("&HTML\tF5"), _("Convert to HTML"));
    convertMoinMenu->Append(ID_CONV_MOIN2BB, _("&bbCode\tF6"), _("Convert to bbCode"));
    convertMenu->Append(wxID_ANY, _("&MoinMoin to..."), convertMoinMenu);

    mbar->Append(convertMenu, _("&Convert"));

    optionsMenu = new wxMenu(_T(""));
    optionsMenu->AppendRadioItem(ID_MENU_NONE, _("&no specific conversion"), _("no specific conversion"));
    optionsMenu->AppendRadioItem(ID_MENU_UU, _("&ubuntuusers-specific conversion"), _("ubuntuusers-specific conversion"));
    optionsMenu->AppendRadioItem(ID_MENU_BLOG, _("&blog-specific conversion"), _("blog-specific conversion"));
    optionsMenu->Check(ID_MENU_UU, true);
    mbar->Append(optionsMenu, _("&Options"));

    wxMenu *helpMenu = new wxMenu(_T(""));
    helpMenu->Append(ID_MENU_ABOUT, _("&About..."), _("Show about dialog"));
    mbar->Append(helpMenu, _("&Help"));

    // layout for the whole frame
    wxBoxSizer *gridSizer = new wxBoxSizer(wxVERTICAL);

    textField=new wxTextCtrl(this, ID_TEXTFIELD, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);

    gridSizer->Add(textField, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 3);

    SetMenuBar(mbar);

    filename.Empty();
    SetSaved(true);
}

ConverterGui::~ConverterGui()
{
}

void ConverterGui::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void ConverterGui::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void ConverterGui::OnTextEnter(wxCommandEvent &event)
{
    if (isSaved)
    {
        SetSaved(false);
    }
}

void ConverterGui::OnOpen(wxCommandEvent &event)
{
    if (!isSaved)
    {
          if (wxMessageBox(_("File is not saved. Do you want to discard changes?"), _("Question: Discard Changes?"), wxICON_QUESTION | wxYES_NO | wxNO_DEFAULT) == wxNO)
        {
            return;
        }
    }

    // file dialog for opening files
    wxFileDialog openFileDialog(this, _("Choose a file"), wxEmptyString, wxEmptyString, _T("*"), wxOPEN | wxCHANGE_DIR);

    if (openFileDialog.ShowModal() == wxID_OK)     // file has been chosen
    {
        filename=openFileDialog.GetPath();
        textField->LoadFile(filename);
        SetSaved(true);
    }

}

void ConverterGui::OnSave(wxCommandEvent &event)
{
    if (!filename.IsEmpty())
    {
        textField->SaveFile(filename);
        SetSaved(true);
    }
   else
    {
        OnSaveAs(event);
    }
}

void ConverterGui::OnSaveAs(wxCommandEvent &event)
{

    // file dialog for opening files
    wxFileDialog saveFileDialog(this, _("Choose a file"), wxEmptyString, wxEmptyString, _T("*"), wxSAVE | wxCHANGE_DIR);

    if (saveFileDialog.ShowModal() == wxID_OK)     // file has been chosen
    {
        filename=saveFileDialog.GetPath();
        OnSave(event);
    }

}

void ConverterGui::SetSaved(const bool saved)
{
    isSaved=saved;

    wxString title;

    if (!filename.IsEmpty())
    {
        title=filename;
    }
    else title=name+_T(" ")+version;

    if (!isSaved)
    {
        title+=_(" [NOT SAVED]");
    }

    SetTitle(title);
}

void ConverterGui::OnAbout(wxCommandEvent &event)
{
    wxString text;
    text << name << _T(" ") << version << _T("\n\n")
         << _T("Copyright (C) 2007,2008 Dominik Wagenfuehr <dominik.wagenfuehr@deesaster.org>\n")
         << _("License: ") << _T("GPLv3");

    wxMessageBox(text, _T("About ")+name);
}

void ConverterGui::ConvertHTML2BB(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertHTML2BB(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertHTML2BB(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertHTML2BB(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

void ConverterGui::ConvertHTML2Moin(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertHTML2Moin(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertHTML2Moin(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertHTML2Moin(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

void ConverterGui::ConvertBB2HTML(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertBB2HTML(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertBB2HTML(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertBB2HTML(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

void ConverterGui::ConvertBB2Moin(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertBB2Moin(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertBB2Moin(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertBB2Moin(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

void ConverterGui::ConvertMoin2HTML(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertMoin2HTML(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertMoin2HTML(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertMoin2HTML(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

void ConverterGui::ConvertMoin2BB(wxCommandEvent &event)
{
    // convert to array
    wxArrayString text;
    for (int i=0; i<textField->GetNumberOfLines(); i++ )
    {
        text.Add(textField->GetLineText(i));
    }

    // create converter
    ConverterBase conv(text);

    // convert text
    if (optionsMenu->IsChecked(ID_MENU_NONE))
    {
        conv.ConvertMoin2BB(ConverterBase::ID_OPTION_NONE);
    }
    else if (optionsMenu->IsChecked(ID_MENU_UU))
    {
        conv.ConvertMoin2BB(ConverterBase::ID_OPTION_UU);
    }
    else if (optionsMenu->IsChecked(ID_MENU_BLOG))
    {
        conv.ConvertMoin2BB(ConverterBase::ID_OPTION_BLOG);
    }

    // get text
    text=conv.GetText();

    // clear text field
    textField->Clear();

    // get text lines
    for (size_t i=0; i<text.GetCount(); i++)
    {
         textField->AppendText(text[i]);
    }

    // set insertion point to start
    textField->SetInsertionPoint(0);

    SetSaved(false);
}

