/*****************************************************************************
 * Name:      convertergui.h
 * Purpose:   Defines Application GUI
 * Created:   2007-04-26
 * Copyright: 2007,2008 Dominik Wagenfuehr <dominik.wagenfuehr@deesaster.org>
 * License:   GPLv3
 *****************************************************************************/

/**
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONVERTERGUI_H
#define CONVERTERGUI_H

#include "converterapp.h"

/// Converter Window
/**
 * Shows a text field for entering text
 * and the menu bar for opening and saving
 * files and converting them.
 */
class ConverterGui: public wxFrame
{
    public:
        /// Constructor
        /**
         * Create window with text field and menu items.
         */
        ConverterGui(wxFrame *frame, const wxString& title, long style = wxDEFAULT_FRAME_STYLE | wxMAXIMIZE);

        /// Destructor.
        /**
         * Automatically frees memory.
         */
        ~ConverterGui();

    private:

        /// Different identifiers.
        enum
        {
            ID_MENU_QUIT = 1000,          ///< Quit program.
            ID_MENU_OPEN,                 ///< Open file.
            ID_MENU_SAVE,                 ///< Save file.
            ID_MENU_SAVE_AS,              ///< Save file as.
            ID_MENU_ABOUT,                ///< About program.
            ID_MENU_NONE,                 ///< no specific conversion.
            ID_MENU_UU,                   ///< ubuntuusers-specific conversion.
            ID_MENU_BLOG,                 ///< own-blog-specific conversion.
            ID_CONV_HTML2BB,              ///< Convert from HTML to MoinMoin.
            ID_CONV_HTML2MOIN,            ///< Convert from HTML to BB-Code.
            ID_CONV_BB2HTML,              ///< Convert from BB-Code to HTML.
            ID_CONV_BB2MOIN,              ///< Convert from BB-Code to MoinMoin.
            ID_CONV_MOIN2HTML,            ///< Convert from MoinMoin to HTML.
            ID_CONV_MOIN2BB,              ///< Convert from  MoinMoin to BB-Code.
            ID_TEXTFIELD,                 ///< Text field.
        };

        /// Close window.
        /**
         * Calls Destroy().
         */
        void OnClose(wxCloseEvent& event);

        /// Quit application.
        /**
         * Calls Destroy().
         */
        void OnQuit(wxCommandEvent& event);

        /// Open file.
        /**
         * If the current file was not saved before
         * the suer will be asked to do so.
         */
        void OnOpen(wxCommandEvent& event);

        /// Save file.
        /**
         * Save file under current filename.
         * If none is given user is asked to
         * select a new one.
         */
        void OnSave(wxCommandEvent& event);

        /// Save file under another filename.
        void OnSaveAs(wxCommandEvent& event);

        /// Show about dialog.
        void OnAbout(wxCommandEvent& event);

        /// Sets the isSaved flag to \b false.
        /**
         * Calls SetSaved(false).
         */
        void OnTextEnter(wxCommandEvent& event);

        /// Sets the isSaved flag.
        void SetSaved(const bool saved);

        /// Convert text from HTML to BB-Code.
        void ConvertHTML2BB(wxCommandEvent& event);

        /// Convert text from HTML to MoinMoin.
        void ConvertHTML2Moin(wxCommandEvent& event);

        /// Convert text from BB-Code to HTML.
        void ConvertBB2HTML(wxCommandEvent& event);

        /// Convert text from BB-Code to BB-Code.
        void ConvertBB2Moin(wxCommandEvent& event);

        /// Convert text from MoinMoin to HTML.
        void ConvertMoin2HTML(wxCommandEvent& event);

        /// Convert text from MoinMoin to BB-Code.
        void ConvertMoin2BB(wxCommandEvent& event);

        DECLARE_EVENT_TABLE()

        wxTextCtrl* textField;     ///< Text field.
        wxMenu* optionsMenu;       ///< Options menu (needed for
        wxString filename;         ///< Current opened file.
        bool isSaved;              ///< Flag if file is saved.
        wxString version;          ///< Current version.
        wxString name;             ///< Program name.
};


#endif // CONVERTERGUI_H
