/*****************************************************************************
 * Name:      converterbase.cpp
 * Purpose:   Converter base class for conversion
 * Created:   2007-04-26
 * Copyright: 2007,2008 Dominik Wagenfuehr <dominik.wagenfuehr@deesaster.org>
 * License:   GPLv3
 *****************************************************************************/

/**
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "converterbase.h"

ConverterBase::ConverterBase(const wxArrayString& text)
{   /// Default Constructor
    m_text=text;
}

const wxArrayString& ConverterBase::GetText(void) const
{   /// Return text file.
    return(m_text);
}

void ConverterBase::Set(const wxArrayString& text)
{   /// Set new text, ignore double line breaks

    // clear old text, keep memory
    m_text.Empty();

    bool linebreak=true;

    for (size_t i=0; i<text.GetCount(); i++)
    {
        if (text[i]!=_T("\n") || !linebreak) m_text.Add(text[i]);
        if (text[i]==_T("\n")) linebreak=true;
        else linebreak=false;
    }

}

void ConverterBase::ConvertHTML2BB(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, userName, wikiName, htmlName;
    wxArrayString newText;
    bool link=false;
    bool userlink=false;
    bool wikilink=false;
    bool list=false;
    bool listentry=false;
    int pos=-1;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // convert links
        size_t j=0;
        bool restart=false;

        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;

            if (tempLine.Lower().StartsWith(_T("<a href=\"")) && link==false)
            {
                newLine+=line.Left(j);             // store first part

                // check what link it is
                tempLine=tempLine.Right(tempLine.Len()-9);  // cut beginning
                line=tempLine;                       // set new line
                restart=true;

                if (option==ID_OPTION_UU && tempLine.StartsWith(_T("http://wiki.ubuntuusers.de/")))
                {
                    tempLine=tempLine.Right(tempLine.Len()-27);
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        wikiName=tempLine.Left(pos);
                        wikilink=true;
                        link=true;
                    }
                }
                else if (option==ID_OPTION_UU && tempLine.StartsWith(_T("http://www.ubuntuusers.de/users/")))
                {
                    tempLine=tempLine.Right(tempLine.Len()-32);
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        userName=tempLine.Left(pos);
                        if (userName.Last()=='/') userName=userName.RemoveLast();
                        userlink=true;
                        link=true;
                    }
                }
                else
                {
                    // normal html link
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        htmlName=tempLine.Left(pos);
                        link=true;
                    }
                }
            }
            else if (tempLine.Lower().StartsWith(_T("</a>")) && link==true)
            {
                line=tempLine.Right(tempLine.Len()-4);
                restart=true;

                if (wikilink==true)
                {
                    if (wikiName==linkName) newLine+=_T("[wiki]")+linkName+_T("[/wiki]");
                    else newLine+=_T("[wiki=")+wikiName+_T("]")+linkName+_T("[/wiki]");
                    wikilink=false;
                }
                else if (userlink==true)
                {
                    userName.Replace(_T("+"), _T(" ")); // replace plus with spaces
                    linkName.Replace(_T("+"), _T(" ")); // replace plus with spaces

                    if (userName==linkName) newLine+=_T("[user]")+linkName+_T("[/user]");
                    else newLine+=_T("[user=")+userName+_T("\"]")+linkName+_T("[/user]");
                    userlink=false;
                }
                else  // standard link
                {
                    if (htmlName==linkName) newLine+=_T("[url]")+linkName+_T("[/url]");
                    else newLine+=_T("[url=")+htmlName+_T("]")+linkName+_T("[/url]");
                }
                link=false;
            }
            else if (tempLine.StartsWith(_T("\">")) && link==true)
            {
                tempLine=tempLine.Right(tempLine.Len()-2);
                pos=tempLine.Lower().First(_T("</a>"));
                if (pos!=-1)
                {
                    linkName=tempLine.Left(pos);
                }
                else linkName=tempLine;
            }
            else if ((tempLine.Lower().StartsWith(_T("<ul>")) || tempLine.Lower().StartsWith(_T("<ol>"))) && !list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                newLine+=_T("[list]");
                list=true;
            }
            else if ((tempLine.Lower().StartsWith(_T("</ul>")) || tempLine.Lower().StartsWith(_T("</ol>"))) && list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);
                line=tempLine;
                restart=true;
                newLine+=_T("[/list]");
                list=false;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("<li>")) && list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                newLine+=_T("[*]");
                listentry=true;
            }
            else if (tempLine.Lower().StartsWith(_T("</li>")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);
                line=tempLine;
                restart=true;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("<br>")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                newLine+=_T("\n");  // insert break
            }
            else if (tempLine.Lower().StartsWith(_T("<br />")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-6);
                line=tempLine;
                restart=true;
                newLine+=_T("\n");  // insert break
            }

            if (restart) j=0;
            else j++;
        } // while j

        newLine+=line;             // store last part

        // headlines
        newLine.Replace(_T("<h1>"), _T("[b]"));
        newLine.Replace(_T("</h1>"), _T("[/b]"));
        newLine.Replace(_T("<h2>"), _T("[b]"));
        newLine.Replace(_T("</h2>"), _T("[/b]"));
        newLine.Replace(_T("<h3>"), _T("[b]"));
        newLine.Replace(_T("</h3>"), _T("[/b]"));
        newLine.Replace(_T("<H1>"), _T("[b]"));
        newLine.Replace(_T("</H1>"), _T("[/b]"));
        newLine.Replace(_T("<H2>"), _T("[b]"));
        newLine.Replace(_T("</H2>"), _T("[/b]"));
        newLine.Replace(_T("<H3>"), _T("[b]"));
        newLine.Replace(_T("</H3>"), _T("[/b]"));

        // ignore breaks
        newLine.Replace(_T("<br>"), _T(""));
        newLine.Replace(_T("<br />"), _T(""));
        newLine.Replace(_T("</br>"), _T(""));
        newLine.Replace(_T("<p>"), _T(""));
        newLine.Replace(_T("</p>"), _T(""));
        newLine.Replace(_T("<BR>"), _T(""));
        newLine.Replace(_T("<BR />"), _T(""));
        newLine.Replace(_T("</BR>"), _T(""));
        newLine.Replace(_T("<P>"), _T(""));
        newLine.Replace(_T("</P>"), _T(""));

        // bold
        newLine.Replace(_T("<b>"), _T("[b]"));
        newLine.Replace(_T("</b>"), _T("[/b]"));
        newLine.Replace(_T("<B>"), _T("[b]"));
        newLine.Replace(_T("</B>"), _T("[/b]"));
        newLine.Replace(_T("<strong>"), _T("[b]"));
        newLine.Replace(_T("</strong>"), _T("[/b]"));
        newLine.Replace(_T("<STRONG>"), _T("[b]"));
        newLine.Replace(_T("</STRONG>"), _T("[/b]"));

        // underline
        newLine.Replace(_T("<u>"), _T("[u]"));
        newLine.Replace(_T("</u>"), _T("[/u]"));
        newLine.Replace(_T("<U>"), _T("[u]"));
        newLine.Replace(_T("</U>"), _T("[/u]"));

        // italic
        newLine.Replace(_T("<i>"), _T("[i]"));
        newLine.Replace(_T("</i>"), _T("[/i]"));
        newLine.Replace(_T("<em>"), _T("[i]"));
        newLine.Replace(_T("</em>"), _T("[/i]"));
        newLine.Replace(_T("<cite>"), _T("[i]"));
        newLine.Replace(_T("</cite>"), _T("[/i]"));
        newLine.Replace(_T("<I>"), _T("[i]"));
        newLine.Replace(_T("</I>"), _T("[/i]"));
        newLine.Replace(_T("<EM>"), _T("[i]"));
        newLine.Replace(_T("</EM>"), _T("[/i]"));
        newLine.Replace(_T("<CITE>"), _T("[i]"));
        newLine.Replace(_T("</CITE>"), _T("[/i]"));

        //code
        newLine.Replace(_T("<code>"), _T("[code]"));
        newLine.Replace(_T("</code>"), _T("[/code]"));
        newLine.Replace(_T("<CODE>"), _T("[code]"));
        newLine.Replace(_T("</CODE>"), _T("[/code]"));

        //special chars
        newLine.Replace(_T("&lt;"), _T("<"));
        newLine.Replace(_T("&gt;"), _T(">"));
        newLine.Replace(_T("&quot;"), _T("\""));
        newLine.Replace(_T("&szlig;"), _T("ß"));
        newLine.Replace(_T("&auml;"), _T("ä"));
        newLine.Replace(_T("&ouml;"), _T("ö"));
        newLine.Replace(_T("&uuml;"), _T("ü"));
        newLine.Replace(_T("&Auml;"), _T("Ä"));
        newLine.Replace(_T("&Ouml;"), _T("Ö"));
        newLine.Replace(_T("&Uuml;"), _T("Ü"));

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("[[en]]"), _T("[flag]en[/flag]"));
        }

        // write new line
        newText.Add(newLine+_T("\n"));
    } // for i

    if (list) // list is still open
    {
        newText.Add(_T("[/list]"));
        list=false;
    }

    Set(newText);

}

void ConverterBase::ConvertHTML2Moin(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, userName, wikiName, htmlName;
    wxArrayString newText;
    bool link=false;
    bool userlink=false;
    bool wikilink=false;
    bool list=false;
    bool listentry=false;
    int pos=-1;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // convert links
        size_t j=0;
        bool restart=false;

        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;

            if (tempLine.Lower().StartsWith(_T("<a href=\"")) && link==false)
            {
                newLine+=line.Left(j);             // store first part

                // check what link it is
                tempLine=tempLine.Right(tempLine.Len()-9);  // cut beginning
                line=tempLine;                       // set new line
                restart=true;

                if (option==ID_OPTION_UU && tempLine.StartsWith(_T("http://wiki.ubuntuusers.de/")))
                {
                    tempLine=tempLine.Right(tempLine.Len()-27);
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        wikiName=tempLine.Left(pos);
                        wikilink=true;
                        link=true;
                    }
                }
                else if (option==ID_OPTION_UU && tempLine.StartsWith(_T("http://www.ubuntuusers.de/users/")))
                {
                    tempLine=tempLine.Right(tempLine.Len()-32);
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        userName=tempLine.Left(pos);
                        if (userName.Last()=='/') userName=userName.RemoveLast();
                        userlink=true;
                        link=true;
                    }
                }
                else
                {
                    // normal html link
                    pos=tempLine.First('\"');
                    if (pos!=-1)
                    {
                        htmlName=tempLine.Left(pos);
                        link=true;
                    }
                }
            }
            else if (tempLine.Lower().StartsWith(_T("</a>")) && link==true)
            {
                line=tempLine.Right(tempLine.Len()-4);
                restart=true;

                if (wikilink==true)
                {
                    if (wikiName==linkName) newLine+=_T("[:")+linkName+_T(":]");
                    else newLine+=_T("[:")+wikiName+_T(":")+linkName+_T("]");
                    wikilink=false;
                }
                else if (userlink==true)
                {
                    userName.Replace(_T("+"), _T(" ")); // replace plus with spaces
                    linkName.Replace(_T("+"), _T(" ")); // replace plus with spaces

                    if (userName==linkName) newLine+=_T("[[user(")+linkName+_T(")]]");
                    else newLine+=_T("[[user(")+linkName+_T(")]]");
                    userlink=false;
                }
                else  // standard link
                {
                    if (htmlName==linkName) newLine+=_T("[")+linkName+_T("]");
                    else newLine+=_T("[")+htmlName+_T(" ")+linkName+_T("]");
                }
                link=false;
            }
            else if (tempLine.StartsWith(_T("\">")) && link==true)
            {
                tempLine=tempLine.Right(tempLine.Len()-2);
                pos=tempLine.Lower().First(_T("</a>"));
                if (pos!=-1)
                {
                    linkName=tempLine.Left(pos);
                }
                else linkName=tempLine;
            }
            else if ((tempLine.Lower().StartsWith(_T("<ul>")) || tempLine.Lower().StartsWith(_T("<ol>"))) && !list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                list=true;
            }
            else if ((tempLine.Lower().StartsWith(_T("</ul>")) || tempLine.Lower().StartsWith(_T("</ol>"))) && list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);
                line=tempLine;
                restart=true;
                list=false;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("<li>")) && list)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                newLine+=_T(" *");
                listentry=true;
            }
            else if (tempLine.Lower().StartsWith(_T("</li>")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);
                line=tempLine;
                restart=true;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("<br>")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);
                line=tempLine;
                restart=true;
                newLine+=_T("\n .");  // insert break
            }
            else if (tempLine.Lower().StartsWith(_T("<br />")) && listentry)
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-6);
                line=tempLine;
                restart=true;
                newLine+=_T("\n .");  // insert break
            }

            if (restart) j=0;
            else j++;

        } // while j

        newLine+=line;             // store last part

        // headlines
        newLine.Replace(_T("<h1>"), _T("= "));
        newLine.Replace(_T("</h1>"), _T(" ="));
        newLine.Replace(_T("<h2>"), _T("== "));
        newLine.Replace(_T("</h2>"), _T(" =="));
        newLine.Replace(_T("<h3>"), _T("=== "));
        newLine.Replace(_T("</h3>"), _T(" ==="));
        newLine.Replace(_T("<H1>"), _T("= "));
        newLine.Replace(_T("</H1>"), _T(" ="));
        newLine.Replace(_T("<H2>"), _T("== "));
        newLine.Replace(_T("</H2>"), _T(" =="));
        newLine.Replace(_T("<H3>"), _T("=== "));
        newLine.Replace(_T("</H3>"), _T(" ==="));

        // ignore breaks
        newLine.Replace(_T("<br>"), _T(""));
        newLine.Replace(_T("<br />"), _T(""));
        newLine.Replace(_T("</br>"), _T(""));
        newLine.Replace(_T("<p>"), _T(""));
        newLine.Replace(_T("</p>"), _T(""));
        newLine.Replace(_T("<BR>"), _T(""));
        newLine.Replace(_T("<BR />"), _T(""));
        newLine.Replace(_T("</BR>"), _T(""));
        newLine.Replace(_T("<P>"), _T(""));
        newLine.Replace(_T("</P>"), _T(""));

        // bold
        newLine.Replace(_T("<b>"), _T("'''"));
        newLine.Replace(_T("</b>"), _T("'''"));
        newLine.Replace(_T("<B>"), _T("'''"));
        newLine.Replace(_T("</B>"), _T("'''"));
        newLine.Replace(_T("<strong>"), _T("'''"));
        newLine.Replace(_T("</strong>"), _T("'''"));
        newLine.Replace(_T("<STRONG>"), _T("'''"));
        newLine.Replace(_T("</STRONG>"), _T("'''"));

        // underline
        newLine.Replace(_T("<u>"), _T("__"));
        newLine.Replace(_T("</u>"), _T("__"));
        newLine.Replace(_T("<U>"), _T("__"));
        newLine.Replace(_T("</U>"), _T("__"));

        // italic
        newLine.Replace(_T("<i>"), _T("''"));
        newLine.Replace(_T("</i>"), _T("''"));
        newLine.Replace(_T("<em>"), _T("''"));
        newLine.Replace(_T("</em>"), _T("''"));
        newLine.Replace(_T("<cite>"), _T("''"));
        newLine.Replace(_T("</cite>"), _T("''"));
        newLine.Replace(_T("<I>"), _T("''"));
        newLine.Replace(_T("</I>"), _T("''"));
        newLine.Replace(_T("<EM>"), _T("''"));
        newLine.Replace(_T("</EM>"), _T("''"));
        newLine.Replace(_T("<CITE>"), _T("''"));
        newLine.Replace(_T("</CITE>"), _T("''"));

        // code
        newLine.Replace(_T("<code>"), _T("`"));
        newLine.Replace(_T("</code>"), _T("`"));
        newLine.Replace(_T("<CODE>"), _T("`"));
        newLine.Replace(_T("</CODE>"), _T("`"));

        // special chars
        newLine.Replace(_T("&lt;"), _T("<"));
        newLine.Replace(_T("&gt;"), _T(">"));
        newLine.Replace(_T("&quot;"), _T("\""));
        newLine.Replace(_T("&szlig;"), _T("ß"));
        newLine.Replace(_T("&auml;"), _T("ä"));
        newLine.Replace(_T("&ouml;"), _T("ö"));
        newLine.Replace(_T("&uuml;"), _T("ü"));
        newLine.Replace(_T("&Auml;"), _T("Ä"));
        newLine.Replace(_T("&Ouml;"), _T("Ö"));
        newLine.Replace(_T("&Uuml;"), _T("Ü"));

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("[[en]]"), _T("{en}"));
        }

        // write new line
        newText.Add(newLine+_T("\n"));

    } // for i

    Set(newText);
}

void ConverterBase::ConvertBB2HTML(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, htmlName, flagLink;
    wxArrayString newText;
    int pos1=-1, pos2=-1;
    bool list=false, listentry=false;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();

        // parentheses
        line.Replace(_T("<"), _T("&lt;"));
        line.Replace(_T(">"), _T("&gt;"));

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // convert links
        size_t j=0;
        bool restart=false;


        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;
            flagLink.Empty();

            if (tempLine.Lower().StartsWith(_T("[url")))
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-4);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/url]"));

                if (! (pos1==-1 || pos2==-1 || pos2<pos1) )
                {
                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/url]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-6);
                    restart=true;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" [flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-16);
                        }
                        else if (line.StartsWith(_T("[flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-15);
                        }
                    }

                    if (htmlName.IsEmpty()) htmlName=linkName;
                    newLine+=_T("<a ")+flagLink+_T("href=\"")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.Lower().StartsWith(_T("[wiki")))
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/wiki]"));

                if (! (pos1==-1 || pos2==-1 || pos2<pos1))
                {
                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/wiki]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-7);
                    restart=true;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" [flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-16);
                        }
                        else if (line.StartsWith(_T("[flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-15);
                        }
                    }

                    if (htmlName.IsEmpty()) htmlName=linkName;

                    htmlName.Replace(_T(" "), _T("_")); // replace spaces with underscores

                    newLine+=_T("<a ")+flagLink+_T("href=\"http://wiki.ubuntuusers.de/")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.Lower().StartsWith(_T("[user")))
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/user]"));

                if (!(pos1==-1 || pos2==-1 || pos2<pos1))
                {
                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/user]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-7);
                    restart=true;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" [flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-16);
                        }
                        else if (line.StartsWith(_T("[flag]en[/flag]")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-15);
                        }
                    }

                    if (htmlName.IsEmpty()) htmlName=linkName;

                    htmlName.Replace(_T(" "), _T("+"));   // replace spaces with plus

                    newLine+=_T("<a ")+flagLink+_T("href=\"http://www.ubuntuusers.de/users/")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.StartsWith(_T("http://")) || tempLine.StartsWith(_T("https://")) || tempLine.StartsWith(_T("ftp://")))
            {
                // search next space
                pos1=tempLine.First(' ');
                if (pos1==-1) pos1=tempLine.Length();

                linkName=tempLine.Left(pos1);
                line=tempLine.Right(tempLine.Len()-pos1);
                restart=true;

                if (option==ID_OPTION_BLOG)
                {
                    if (line.StartsWith(_T(" [flag]en[/flag]")))
                    {
                        flagLink=_T("class=\"english\" ");
                        line=line.Right(line.Len()-16);
                    }
                    else if (line.StartsWith(_T("[flag]en[/flag]")))
                    {
                        flagLink=_T("class=\"english\" ");
                        line=line.Right(line.Len()-15);
                    }
                }

                newLine+=_T("<a  ")+flagLink+_T("href=\"")+linkName+_T("\">")+linkName+_T("</a>");
            }
            else if (tempLine.Lower().StartsWith(_T("[list]")) && !list)
            {
                newLine+=line.Left(j);             // store first part
                newLine+=_T("<ul>");
                line=tempLine.Right(tempLine.Len()-6);
                restart=true;

                list=true;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("[/list]")) && list)
            {
                newLine+=line.Left(j);             // store first part
                line=tempLine.Right(tempLine.Len()-7);
                restart=true;

                list=false;
                if (listentry)
                {
                    newLine+=_T("</li>"); // end list entry
                    listentry=false;
                }
                newLine+=_T("</ul>");

            }
            else if (tempLine.StartsWith(_T("[*]")) && list)
            {
                newLine+=line.Left(j);             // store first part
                line=tempLine.Right(tempLine.Len()-3);
                restart=true;

                if (listentry)
                {
                    newLine+=_T("</li>"); // end list entry
                    listentry=false;
                }

                listentry=true;  // start new entry
                newLine+=_T("<li>"); // start list entry
            }

            if (restart) j=0;
            else j++;
        } // while j

        newLine+=line;             // store last part
        if (list && listentry)
        {
            newLine+=_T("</li>"); // end list entry
            listentry=false;
        }

        // bold
        newLine.Replace(_T("[b]"), _T("<b>"));
        newLine.Replace(_T("[/b]"), _T("</b>"));
        newLine.Replace(_T("[B]"), _T("<b>"));
        newLine.Replace(_T("[/B]"), _T("</b>"));

        // underline
        newLine.Replace(_T("[u]"), _T("<u>"));
        newLine.Replace(_T("[/u]"), _T("</u>"));
        newLine.Replace(_T("[U]"), _T("<u>"));
        newLine.Replace(_T("[/U]"), _T("</u>"));

        // italic
        newLine.Replace(_T("[i]"), _T("<i>"));
        newLine.Replace(_T("[/i]"), _T("</i>"));
        newLine.Replace(_T("[I]"), _T("<i>"));
        newLine.Replace(_T("[/I]"), _T("</i>"));

        // code
        newLine.Replace(_T("[code]"), _T("<code>"));
        newLine.Replace(_T("[/code]"), _T("</code>"));
        newLine.Replace(_T("[CODE]"), _T("<code>"));
        newLine.Replace(_T("[/CODE]"), _T("</code>"));

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("[flag]en[/flag]"), _T("[[en]]"));
        }

        // write new line
        newText.Add(newLine+_T("\n"));
    } // for i

    Set(newText);
}

void ConverterBase::ConvertBB2Moin(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, htmlName;
    wxArrayString newText;
    int pos1=-1, pos2=-1;
    bool list=false, listentry=false;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // convert links
        size_t j=0;
        bool restart=false;

        if (list && listentry)
        {
            if (!line.StartsWith(_T("[*]")) && !line.StartsWith(_T("[/list")))
            {
                newLine+=(_T(" ."));
            }
        }

        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;

            if (tempLine.Lower().StartsWith(_T("[url")))
            {
                tempLine=tempLine.Right(tempLine.Len()-4);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/url]"));

                if (! (pos1==-1 || pos2==-1 || pos2<pos1 || (pos1>0 && !tempLine.StartsWith(_T("=")))) )
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/url]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-6);
                    restart=true;

                    if (htmlName.IsEmpty()) newLine+=_T("[")+linkName+_T("]");
                    else newLine+=_T("[")+htmlName+_T(" ")+linkName+_T("]");
                }
            }
            else if (tempLine.Lower().StartsWith(_T("[wiki")))
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/wiki]"));

                if (!(pos1==-1 || pos2==-1 || pos2<pos1))
                {
                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/wiki]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-7);
                    restart=true;

                    if (option==ID_OPTION_UU)
                    {
                        if (htmlName.IsEmpty()) newLine+=_T("[:")+linkName+_T(":]");
                        else newLine+=_T("[:")+htmlName+_T(":")+linkName+_T("]");
                    }
                    else
                    {
                        if (htmlName.IsEmpty()) newLine+=_T("[http://wiki.ubuntuusers.de/")+linkName+_T("]");
                        else newLine+=_T("[http://wiki.ubuntuusers.de/")+htmlName+_T(" ")+linkName+_T("]");
                    }
                }
            }
            else if (tempLine.Lower().StartsWith(_T("[user")))
            {
                newLine+=line.Left(j);             // store first part
                tempLine=tempLine.Right(tempLine.Len()-5);

                // check if other linkname
                pos1=tempLine.First(']');
                pos2=tempLine.Lower().Find(_T("[/user]"));

                if (!(pos1==-1 || pos2==-1 || pos2<pos1))
                {
                    htmlName=tempLine.Left(pos1);
                    htmlName.Replace(_T("="),_T(""), false);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.Lower().Find(_T("[/user]"));

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-7);
                    restart=true;

                    if (option==ID_OPTION_UU)
                    {
                        newLine+=_T("[[user(")+linkName+_T(")]]");
                    }
                    else
                    {
                        if (htmlName.IsEmpty()) htmlName=linkName;

                        htmlName.Replace(_T(" "), _T("+"));   // replace spaces with plus
                        newLine+=_T("[http://www.ubuntuusers.de/users/")+htmlName+_T(" ")+linkName+_T("]");
                    }
                }
            }
            else if (tempLine.StartsWith(_T("http://")) || tempLine.StartsWith(_T("https://")) || tempLine.StartsWith(_T("ftp://")))
            {
                // search next space
                pos1=tempLine.First(' ');
                if (pos1==-1) pos1=tempLine.Length();

                linkName=tempLine.Left(pos1);
                line=tempLine.Right(tempLine.Len()-pos1);
                restart=true;

                newLine+=_T("[")+linkName+_T("]");
            }
            else if (tempLine.Lower().StartsWith(_T("[list]")) && !list)
            {
                newLine+=line.Left(j);             // store first part
                line=tempLine.Right(tempLine.Len()-6);
                restart=true;

                list=true;
                listentry=false;
            }
            else if (tempLine.Lower().StartsWith(_T("[/list]")) && list)
            {
                newLine+=line.Left(j);             // store first part
                line=tempLine.Right(tempLine.Len()-7);
                restart=true;
                list=false;
                listentry=false;
            }
            else if (tempLine.StartsWith(_T("[*]")) && list)
            {
                newLine+=line.Left(j);             // store first part
                line=tempLine.Right(tempLine.Len()-3);
                restart=true;

                if (j!=0)
                {
                    newLine+=_T("\n"); // end list entry
                    listentry=false;
                }

                listentry=true;  // start new entry
                newLine+=_T(" *"); // start list entry
            }

            if (restart) j=0;
            else j++;
        } // while j

        newLine+=line;             // store last part

        // bold
        newLine.Replace(_T("[b]"), _T("'''"));
        newLine.Replace(_T("[/b]"), _T("'''"));
        newLine.Replace(_T("[B]"), _T("'''"));
        newLine.Replace(_T("[/B]"), _T("'''"));

        // underline
        newLine.Replace(_T("[u]"), _T("__"));
        newLine.Replace(_T("[/u]"), _T("__"));
        newLine.Replace(_T("[U]"), _T("__"));
        newLine.Replace(_T("[/U]"), _T("__"));

        // italic
        newLine.Replace(_T("[i]"), _T("''"));
        newLine.Replace(_T("[/i]"), _T("''"));
        newLine.Replace(_T("[I]"), _T("''"));
        newLine.Replace(_T("[/I]"), _T("''"));

        // code
        newLine.Replace(_T("[code]"), _T("`"));
        newLine.Replace(_T("[/code]"), _T("`"));
        newLine.Replace(_T("[CODE]"), _T("`"));
        newLine.Replace(_T("[/CODE]"), _T("`"));

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("[flag]en[/flag]"), _T("{en}"));
        }

        // write new line
        newText.Add(newLine+_T("\n"));

    } // for i

    Set(newText);
}

void ConverterBase::ConvertMoin2HTML(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, htmlName, flagLink;
    wxArrayString newText;
    bool list=false;
    bool listentry=false;
    bool bold=false;
    bool italic=false;
    bool underline=false;
    bool code=false;
    int pos1=-1, pos2=-1;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();
        // listentry=false;

        // parentheses
        line.Replace(_T("<"), _T("&lt;"));
        line.Replace(_T(">"), _T("&gt;"));

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // set lists
        if (line.StartsWith(_T(" *")))
        {
            if (!list) newText.Add(_T("<ul>\n"));
            list=true;
            listentry=true;
            newLine+=_T("<li>");
            line.Replace(_T(" *"), _T(""), false);
        }
        else if (line.StartsWith(_T(" .")))
        {
            if (!list) newText.Add(_T("<ul>\n"));
            if (!listentry) newLine+=_T("<li>");
            else newLine+=_T("<br />");
            list=true;
            listentry=true;
            line.Replace(_T(" ."), _T(""), false);
        }
        else if (list)
        {
            newText.Add(_T("</ul>\n"));
            list=false;
            listentry=false;
        }

        if (line.StartsWith(_T("=== ")) && line.Find(_T(" ==="))>-1)
        {
            line=_T("<h3>")+line.Mid(4,line.Length()-8)+_T("</h3>");
        }
        else if(line.StartsWith(_T("== ")) && line.Find(_T(" =="))>-1)
        {
            line=_T("<h2>")+line.Mid(3,line.Length()-6)+_T("</h2>");
        }
        else if(line.StartsWith(_T("= ")) && line.Find(_T(" ="))>-1)
        {
            line=_T("<h1>")+line.Mid(2,line.Length()-4)+_T("</h1>");
        }

        // convert links
        size_t j=0;
        bool restart=false;

        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;
            flagLink.Empty();

            if (tempLine.StartsWith(_T("[:")))
            {
                tempLine=tempLine.Right(tempLine.Len()-2);

                // check if other linkname
                pos1=tempLine.First(':');
                pos2=tempLine.First(']');

                if (! (pos1==-1 || pos2==-1 || pos2<pos1) )
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.First(']');

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-1);
                    restart=true;

                    if (linkName.IsEmpty()) linkName=htmlName;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" {en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-5);
                        }
                        else if (line.StartsWith(_T("{en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-4);
                        }
                    }

                    htmlName.Replace(_T(" "), _T("_"));   // replace spaces with underscores

                    newLine+=_T("<a ")+flagLink+_T("href=\"http://wiki.ubuntuusers.de/")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.StartsWith(_T("[\"")))
            {
                tempLine=tempLine.Right(tempLine.Len()-2);

                // check if other linkname
                pos1=tempLine.Find(_T("\"]"));

                if (pos1!=-1)
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);
                    linkName=htmlName;

                    line=tempLine.Right(tempLine.Len()-pos1-2);
                    restart=true;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" {en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-5);
                        }
                        else if (line.StartsWith(_T("{en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-4);
                        }
                    }

                    htmlName.Replace(_T(" "), _T("_"));   // replace spaces with underscores

                    newLine+=_T("<a ")+flagLink+_T("href=\"http://wiki.ubuntuusers.de/")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.StartsWith(_T("[[user(")))
            {
                tempLine=tempLine.Right(tempLine.Len()-7);

                // check if other linkname
                pos1=tempLine.Find(_T(")]]"));

                if (pos1!=-1)
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);
                    linkName=htmlName;

                    line=tempLine.Right(tempLine.Len()-pos1-3);
                    restart=true;

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" {en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-5);
                        }
                        else if (line.StartsWith(_T("{en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-4);
                        }
                    }

                    htmlName.Replace(_T(" "), _T("+"));   // replace spaces with plus

                    newLine+=_T("<a ")+flagLink+_T("href=\"http://www.ubuntuusers.de/users/")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.StartsWith(_T("[http://")) || tempLine.StartsWith(_T("[https://")) || tempLine.StartsWith(_T("[ftp://")))
            {
                tempLine=tempLine.Right(tempLine.Len()-1);

                // search last bracket
                pos1=tempLine.First(' ');
                pos2=tempLine.First(']');

                if ( pos2!=-1 )
                {
                    if (pos1==-1 || pos1>pos2)  // no alternate ename
                    {
                        newLine+=line.Left(j);             // store first part

                        linkName=tempLine.Left(pos2);
                        htmlName=linkName;

                        line=tempLine.Right(tempLine.Len()-pos2-1);
                        restart=true;
                    }
                    else
                    {
                        newLine+=line.Left(j);             // store first part

                        htmlName=tempLine.Left(pos1);

                        line=tempLine.Right(tempLine.Len()-pos1-1);
                        tempLine=line;

                        pos2=tempLine.First(']');

                        linkName=tempLine.Left(pos2);
                        line=tempLine.Right(tempLine.Len()-pos2-1);
                        restart=true;
                    }

                    if (option==ID_OPTION_BLOG)
                    {
                        if (line.StartsWith(_T(" {en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-5);
                        }
                        else if (line.StartsWith(_T("{en}")))
                        {
                            flagLink=_T("class=\"english\" ");
                            line=line.Right(line.Len()-4);
                        }
                    }

                    newLine+=_T("<a ")+flagLink+_T("href=\"")+htmlName+_T("\">")+linkName+_T("</a>");
                }
            }
            else if (tempLine.StartsWith(_T("http://")) || tempLine.StartsWith(_T("https://")) || tempLine.StartsWith(_T("ftp://")))
            {
                // search next space
                pos1=tempLine.First(' ');
                if (pos1==-1) pos1=tempLine.Len();

                linkName=tempLine.Left(pos1);
                htmlName=linkName;

                line=tempLine.Right(tempLine.Len()-pos1);
                restart=true;

                if (option==ID_OPTION_BLOG)
                {
                    if (line.StartsWith(_T(" {en}")))
                    {
                        flagLink=_T("class=\"english\" ");
                        line=line.Right(line.Len()-5);
                    }
                    else if (line.StartsWith(_T("{en}")))
                    {
                        flagLink=_T("class=\"english\" ");
                        line=line.Right(line.Len()-4);
                    }
                }

                newLine+=_T("<a ")+flagLink+_T("href=\"")+htmlName+_T("\">")+linkName+_T("</a>");
            }
            else if (tempLine.StartsWith(_T("'''")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-3);
                restart=true;

                if (!bold) newLine+=_T("<b>");
                else newLine+=_T("</b>");
                bold=!bold;
            }
            else if (tempLine.StartsWith(_T("''")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-2);
                restart=true;

                if (!italic) newLine+=_T("<i>");
                else newLine+=_T("</i>");
                italic=!italic;
            }
            else if (tempLine.StartsWith(_T("__")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-2);
                restart=true;

                if (!underline) newLine+=_T("<u>");
                else newLine+=_T("</u>");
                underline=!underline;
            }
            else if (tempLine.StartsWith(_T("`")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-1);
                restart=true;

                if (!code) newLine+=_T("<code>");
                else newLine+=_T("</code>");
                code=!code;
            }

            if (restart) j=0;
            else j++;
        } // while j

        newLine+=line;             // store last part

        // close list entry
        if (listentry)
        {
            newLine+=_T("</li>");
            listentry=false;
        }

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("{en}"), _T("[[en]]"));
        }

        // replace [[BR]] with newlines
        newLine.Replace(_T("[[BR]]"), _T("\n"));

        // write new line
        newText.Add(newLine+_T("\n"));
    } // for i

    // list is still open
    if (list)
    {
        newText.Add(_T("</ul>\n"));
        list=false;
    }

    Set(newText);
}

void ConverterBase::ConvertMoin2BB(const OptionID option)
{
    wxString line, tempLine, newLine, linkName, htmlName;
    wxArrayString newText;
    bool list=false, listentry=false;
    bool bold=false;
    bool italic=false;
    bool underline=false;
    bool code=false;
    int pos1=-1, pos2=-1;

    // iterate over all lines
    for (size_t i=0; i<m_text.GetCount(); i++)
    {
        line=m_text[i];
        newLine.Empty();

        // 02.07.2007: wxHack because there is an error in wx-versions > 2.6.1 and < 2.8.4
        if (!line.IsEmpty() && line[0]=='\n')
        {
            newText.Add(_T("\n"));
            line=line.Right(line.Len()-1);
            i++;
        }

        // set lists
        if (line.StartsWith(_T(" *")))
        {
            if (!list) newText.Add(_T("[list]\n"));
            list=true;
            listentry=true;
            newLine+=_T("[*]");
            line.Replace(_T(" *"), _T(""), false);
        }
        else if (line.StartsWith(_T(" .")))
        {
            if (!list) newText.Add(_T("[list]\n"));
            if (!listentry) newLine+=(_T("\n"));
            line.Replace(_T(" ."), _T(""), false);
        }
        else if (list)
        {
            newText.Add(_T("[/list]\n"));
            list=false;
            listentry=false;
        }

        if (line.StartsWith(_T("=== ")) && line.Find(_T(" ==="))>-1)
        {
            line=_T("[b]")+line.Mid(4,line.Length()-8)+_T("[/b]");
        }
        else if(line.StartsWith(_T("== ")) && line.Find(_T(" =="))>-1)
        {
            line=_T("[b]")+line.Mid(3,line.Length()-6)+_T("[/b]");
        }
        else if(line.StartsWith(_T("= ")) && line.Find(_T(" ="))>-1)
        {
            line=_T("[b]")+line.Mid(2,line.Length()-4)+_T("[/b]");
        }

        // convert links
        size_t j=0;
        bool restart=false;

        while (j<line.Len())
        {
            // cut from right
            tempLine=line.Right(line.Len()-j);
            restart=false;

            if (tempLine.StartsWith(_T("[:")))
            {
                tempLine=tempLine.Right(tempLine.Len()-2);

                // check if other linkname
                pos1=tempLine.First(':');
                pos2=tempLine.First(']');

                if (! (pos1==-1 || pos2==-1 || pos2<pos1) )
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);

                    line=tempLine.Right(tempLine.Len()-pos1-1);
                    tempLine=line;

                    pos2=tempLine.First(']');

                    linkName=tempLine.Left(pos2);
                    line=tempLine.Right(tempLine.Len()-pos2-1);
                    restart=true;

                    if (option==ID_OPTION_UU)
                    {
                        if (linkName.IsEmpty()) newLine+=_T("[wiki]")+htmlName+_T("[/wiki]");
                        else newLine+=_T("[wiki=")+htmlName+_T("]")+linkName+_T("[/wiki]");
                    }
                    else
                    {
                        htmlName.Replace(_T(" "), _T("_"));   // replace spaces with underscores

                        if (linkName.IsEmpty()) newLine+=_T("[url]http://wiki.ubuntuusers.de/")+htmlName+_T("[/url]");
                        else newLine+=_T("[url=http://wiki.ubuntuusers.de/")+htmlName+_T("]")+linkName+_T("[/url]");
                    }
                }
            }
            else if (tempLine.StartsWith(_T("[\"")))
            {
                tempLine=tempLine.Right(tempLine.Len()-2);

                // check if other linkname
                pos1=tempLine.Find(_T("\"]"));

                if (pos1!=-1)
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);

                    line=tempLine.Right(tempLine.Len()-pos1-2);
                    restart=true;

                    if (option==ID_OPTION_UU)
                    {
                        newLine+=_T("[wiki]")+htmlName+_T("[/wiki]");
                    }
                    else
                    {
                        htmlName.Replace(_T(" "), _T("_"));   // replace spaces with underscores
                        newLine+=_T("[url]http://wiki.ubuntuusers.de/")+htmlName+_T("[/url]");
                    }
                }
            }
            else if (tempLine.StartsWith(_T("[[user(")))
            {
                tempLine=tempLine.Right(tempLine.Len()-7);

                // check if other linkname
                pos1=tempLine.Find(_T(")]]"));

                if (pos1!=-1)
                {
                    newLine+=line.Left(j);             // store first part

                    htmlName=tempLine.Left(pos1);

                    line=tempLine.Right(tempLine.Len()-pos1-3);
                    restart=true;

                    if (option==ID_OPTION_UU)
                    {
                        newLine+=_T("[user]")+htmlName+_T("[/user]");
                    }
                    else
                    {
                        linkName=htmlName;
                        htmlName.Replace(_T(" "), _T("+"));   // replace spaces with plus

                        newLine+=_T("[url=http://www.ubuntuusers.de/users/")+htmlName+_T("/]")+linkName+_T("[/url]");
                    }

                }
            }
            else if (tempLine.StartsWith(_T("[http://")) || tempLine.StartsWith(_T("[https://")) ||  tempLine.StartsWith(_T("[ftp://")))
            {
                tempLine=tempLine.Right(tempLine.Len()-1);

                // search last bracket
                pos1=tempLine.First(' ');
                pos2=tempLine.First(']');

                if ( pos2!=-1 )
                {
                    if (pos1==-1 || pos1>pos2)  // no alternate ename
                    {
                        newLine+=line.Left(j);             // store first part

                        htmlName=tempLine.Left(pos2);

                        line=tempLine.Right(tempLine.Len()-pos2-1);
                        restart=true;

                        newLine+=_T("[url]")+htmlName+_T("[/url]");
                    }
                    else
                    {
                        newLine+=line.Left(j);             // store first part

                        htmlName=tempLine.Left(pos1);

                        line=tempLine.Right(tempLine.Len()-pos1-1);
                        tempLine=line;

                        pos2=tempLine.First(']');

                        linkName=tempLine.Left(pos2);
                        line=tempLine.Right(tempLine.Len()-pos2-1);
                        restart=true;

                        newLine+=_T("[url=")+htmlName+_T("]")+linkName+_T("[/url]");
                    }
                }
            }
            else if (tempLine.StartsWith(_T("http://")) || tempLine.StartsWith(_T("https://")) || tempLine.StartsWith(_T("ftp://")))
            {
                // search next space
                pos1=tempLine.First(' ');
                if (pos1==-1) pos1=tempLine.Len();

                htmlName=tempLine.Left(pos1);

                line=tempLine.Right(tempLine.Len()-pos1);
                restart=true;

                newLine+=_T("[url]")+htmlName+_T("[/url]");
            }
            else if (tempLine.StartsWith(_T("'''")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-3);
                restart=true;

                if (!bold) newLine+=_T("[b]");
                else newLine+=_T("[/b]");
                bold=!bold;
            }
            else if (tempLine.StartsWith(_T("''")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-2);
                restart=true;

                if (!italic) newLine+=_T("[i]");
                else newLine+=_T("[/i]");
                italic=!italic;
            }
            else if (tempLine.StartsWith(_T("__")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-2);
                restart=true;

                if (!underline) newLine+=_T("[u]");
                else newLine+=_T("[/u]");
                underline=!underline;
            }
            else if (tempLine.StartsWith(_T("`")))
            {
                newLine+=line.Left(j);             // store first part

                line=tempLine.Right(tempLine.Len()-1);
                restart=true;

                if (!code) newLine+=_T("[code]");
                else newLine+=_T("[/code]");
                code=!code;
            }
            if (restart) j=0;
            else j++;
        } // while j

        newLine+=line;             // store last part

        // flags
        if (option==ID_OPTION_UU)
        {
            newLine.Replace(_T("{en}"), _T("[flag]en[/flag]"));
        }

        // replace [[BR]] with newlines
        newLine.Replace(_T("[[BR]]"), _T("\n"));

        // write new line
        newText.Add(newLine+_T("\n"));
    } // for i

    // list is still open
    if (list)
    {
        newText.Add(_T("</ul>\n"));
        list=false;
    }

    Set(newText);
}

